package com.rnalice

import TransformationTools.Companion.stringFromSandboxError
import android.app.Activity
import android.content.Intent
import android.util.Log
import com.alicebiometrics.onboarding.auth.Response
import com.alicebiometrics.onboarding.auth.SandboxAuthenticator
import com.facebook.react.bridge.*
import com.alicebiometrics.onboarding.sandbox.*
import com.facebook.react.bridge.ReactMethod
import com.facebook.react.bridge.Promise
import org.json.JSONObject



class AliceModule: ReactContextBaseJavaModule, ActivityEventListener {
    private val ALICE_ACTIVITY_REQUEST = 220
    private var mPromise: Promise? = null
    private val E_ACTIVITY_DOES_NOT_EXIST = "E_ACTIVITY_DOES_NOT_EXIST"

    constructor(reactContext: ReactApplicationContext): super(reactContext) {
        reactContext.addActivityEventListener(this)
    }

    override fun getName(): String {
        return "Alice"
    }

    @ReactMethod
    fun logInWithSandbox(sandboxToken: String, email: String, firstName: String = "", lastName: String = "", promise: Promise) {
        mPromise = promise
        if(!isValidEmail(email)) {
            mPromise!!.reject("error","invalid email")
            return
            }

        val userInfo = UserInfo(
                email = email,
                firstName = firstName,
                lastName = lastName
        )

        val authenticator = SandboxAuthenticator(
                sandboxToken = sandboxToken,
                userInfo = userInfo
        )

        authenticator.execute { response ->
            when (response) {
                is Response.Success -> mPromise!!.resolve(response.message)
                is Response.Failure -> mPromise!!.reject("error", stringFromSandboxError(response.error))
            }
        }
    }

    @ReactMethod
    fun runOnboarding(userToken: String, config: ReadableMap, promise: Promise) {
        val currentActivity = currentActivity

        mPromise = promise

        if (currentActivity == null) {
            promise.reject(E_ACTIVITY_DOES_NOT_EXIST, "Activity doesn't exist")
            return
        }

        val intent = Intent(currentActivity, AliceActivity::class.java)
        intent.putExtra("userToken", userToken)
        intent.putExtra("config", config.toHashMap())

        currentActivity.startActivityForResult(intent, ALICE_ACTIVITY_REQUEST)
    }

    override fun onNewIntent(intent: Intent?) {
        Log.i("ONBOARDING", "onNewIntent")
    }

    override fun onActivityResult(activity: Activity?, requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == ALICE_ACTIVITY_REQUEST) {
            if (resultCode == Activity.RESULT_OK) {
                val userStatusContent = data!!.getStringExtra("userStatus")
                val userStatus = JSONObject()
                userStatus.put("userStatus",  JSONObject(userStatusContent))
                mPromise!!.resolve(userStatus.toString())
            } else if (resultCode == Activity.RESULT_CANCELED) {
                mPromise!!.reject("error", "cancel")
            }
        }
    }

    fun isValidEmail(email: String): Boolean {
        return android.util.Patterns.EMAIL_ADDRESS.matcher(email).matches()
    }
}
