//  Alice.swift
//  ALiCE Biometrics
//  Copyright © 2020+ ALiCE Biometrics. All rights reserved.


import Foundation
import AliceOnboarding
import AVFoundation
import React

@objc(Alice)
class Alice: NSObject {
    
    @objc(runOnboarding:config:resolve:reject:)
    func runOnboarding(userToken:String,
                       config:NSDictionary,
                       resolve: @escaping RCTPromiseResolveBlock,
                       reject: @escaping RCTPromiseRejectBlock) {

        let configDictionary = getConfigDictionary(nsDictionaryConfig: config)

        AVCaptureDevice.requestAccess(for: AVMediaType.video) { response in
            if response {
                DispatchQueue.main.async { () -> Void in
                    let rootViewController = UIApplication.shared.windows.first!.rootViewController!
                    onboardingBindingFromDictionary(
                        rootViewController,
                        userToken: userToken,
                        onboardingConfigDictionary: configDictionary
                    ){ result in
                        switch result {
                        case let .success(userStatus):
                            resolve(String(describing: userStatus))
                        case let .failure(error):
                            reject("error", stringFromOnboardingError(error), error)
                        case .cancel:
                            reject("error", "cancel", nil)
                        }
                    }
                }
            } else {
                reject("error", "camera permissions not granted", nil)
            }
        }
    }
    
    func isValidEmail(email: String) -> Bool {
        let emailRegEx = "[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,64}"
        let emailPred = NSPredicate(format: "SELF MATCHES %@", emailRegEx)
        return emailPred.evaluate(with: email)
    }
    
    @objc(logInWithSandbox:email:firstName:lastName:resolve:reject:)
    func logInWithSandbox(sandboxToken: String,
                          email: String,
                          firstName: String? = "",
                          lastName: String? = "",
                          resolve: @escaping RCTPromiseResolveBlock,
                          reject: @escaping RCTPromiseRejectBlock) {
        
        if isValidEmail(email: email) {
            let sandboxManager = SandboxManager(sandboxToken: sandboxToken)
            sandboxManager.getUser(email: email) { resultGetUser in
                if case .success = resultGetUser {
                    sandboxManager.getUserToken(email: email) { resultGetUserToken in
                        switch resultGetUserToken {
                        case .success(let userToken):
                            resolve(userToken)
                        case .failure(let error):
                            reject("error", stringFromAuthenticationError(error), error)
                        }
                    }
                } else {
                    let userInfo = UserInfo(email: email,
                                            firstName: firstName!,
                                            lastName: lastName!)
                    sandboxManager.createUser(userInfo: userInfo) { resultCreateUser in
                        switch resultCreateUser {
                        case .success:
                            sandboxManager.getUserToken(email: email) { resultGetUserToken in
                                switch resultGetUserToken {
                                case .success(let userToken):
                                    resolve(userToken)
                                    //completion(.success(userToken))
                                case .failure(let error):
                                    reject("error", stringFromAuthenticationError(error), error)
                                }
                            }
                        case .failure(let error):
                            reject("error", stringFromAuthenticationError(error), error)
                        }
                    }
                }
            }
        } else {
            reject("error", "invalid email", nil)
        }
    }
}
